package se.su.dsv.scipro.reviewing;

import com.google.common.base.Predicate;
import com.google.common.collect.FluentIterable;
import lombok.Getter;
import scala.Option;
import se.su.dsv.scipro.file.FileDescription;
import se.su.dsv.scipro.misc.DaysService;
import se.su.dsv.scipro.project.Project;
import se.su.dsv.scipro.system.DomainObject;

import javax.persistence.*;
import java.util.Collections;
import java.util.Date;
import java.util.LinkedList;
import java.util.List;

@Entity
@DiscriminatorColumn(name = "type", length = 64)
@SuppressWarnings("PMD.UnusedPrivateField")
public abstract class ReviewerApproval extends DomainObject {
    @OneToOne(optional = false)
    protected Project project;

    @OneToMany(mappedBy = "reviewerApproval", cascade = CascadeType.ALL)
    @OrderBy("requested desc")
    protected List<Decision> decisions = new LinkedList<>();

    @Id
    @GeneratedValue
    @Getter
    private Long id;

    public abstract Step getStep();

    public FileDescription getCurrentThesis() {
        return getCurrentDecision().getThesis();
    }

    public Project getProject(){return this.project;}



    public Status getCurrentStatus() {
        return getCurrentDecision().getStatus();
    }

    public boolean isDecided() {
        return getCurrentStatus() != Status.UNDECIDED;
    }

    public String getCurrentReason() {
        return getCurrentDecision().getReason();
    }

    public Option<FileDescription> getCurrentAttachment() {
        return getCurrentDecision().getAttachment();
    }

    public Option<FileDescription> getCurrentGradingReport() {
        return getCurrentDecision().getGradingReport();
    }

    public void approve(final String reason, final Option<FileDescription> attachment, final Option<FileDescription> gradingReport) {
        getCurrentDecision().approve(reason, attachment, gradingReport);
    }

    public void reject(final String reason, final Option<FileDescription> attachment, final Option<FileDescription> gradingReport) {
        getCurrentDecision().reject(reason, attachment, gradingReport);
    }

    public Decision getCurrentDecision() {
        return decisions.get(0);
    }

    public Status getCurrentDecisionStatus() {
        return decisions.get(0).getStatus();
    }

    public void addNewThesis(final FileDescription thesis, final String comment) {
        if (getCurrentStatus() != Status.REJECTED) {
            throw new IllegalStateException();
        }
        this.decisions.add(0, new Decision(this, thesis, comment));
    }

    public List<Decision> getHistory() {
        return Collections.unmodifiableList(decisions);
    }

    public List<Decision> getLateDecisions(final DaysService daysService, final ReviewerDeadlineSettings reviewerDeadlineSettings) {
        return FluentIterable.from(decisions)
                .filter(new Predicate<Decision>() {
                    @Override
                    public boolean apply(final Decision input) {
                        return input.isLate(daysService, getDaysToDecide(reviewerDeadlineSettings));
                    }
                })
                .toList();
    }

    public boolean isRejected() {
        return getCurrentStatus() == Status.REJECTED;
    }

    public enum Step {
        ROUGH_DRAFT_APPROVAL,
        FINAL_SEMINAR_APPROVAL
    }

    public Date getCurrentDeadline(final DaysService daysService, ReviewerDeadlineSettings reviewerDeadlineSettings) {
        return getCurrentDecision().getDeadline(daysService, getDaysToDecide(reviewerDeadlineSettings)).toDate();
    }

    protected abstract int getDaysToDecide(final ReviewerDeadlineSettings reviewerDeadlineSettings);

}
