package se.su.dsv.scipro.configuration;

import javax.xml.XMLConstants;
import javax.xml.namespace.NamespaceContext;
import java.util.*;

/**
 * Utility class for providing configurable namespace contexts, satisfies specification defined in the NamespaceContext interface.
 * The default namespace-prefixes (DEFAULT_NS_PREFIX, XML_NS_PREFIX and XMLNS_ATTRIBUTE) are automatically mapped to defaults and are not configurable from clients.
 * Typical usage:
 * <code>
 * 	MutableNamespaceContext mnc = new MutableNamespaceContext();
 * 	mnc.setMapping("somePrefix", "someNamespaceURI");
 * 	XPath xp = .....
 *  xp.setNamespaceContext(mnc);
 * </code>
 *
 */
public final class MutableNamespaceContext implements NamespaceContext {
	private final Map<String,String> nsMap = new HashMap<>();
	/**
	 * Default constructor.
	 */
	public MutableNamespaceContext(){
		setupDefaults();
	}
	/**
	 * Attempts to resolve the given prefix into a namespaceURI.
	 * @return The namespace URI or XMLConstants.NULL_NS_URI if no mapping is found.
	 * @throws IllegalStateException if the given prefix is null.
	 */
	@Override
	public String getNamespaceURI(String prefix){
		if(prefix == null) {
            throw new IllegalStateException("A null prefix not allowed");
        }
		if(!nsMap.containsKey(prefix)) {
            return XMLConstants.NULL_NS_URI;
        } else {
            return nsMap.get(prefix);
        }
	}
	/**
	 * Resolves all registered prefixes for the given URI.
	 * @param URI
	 * @return An iterator of all registered prefixes for the given URI, the empty iterator if none are found.
	 */
	@SuppressWarnings("unchecked")
	@Override
    public Iterator<String> getPrefixes(String URI) {
		if(URI == null) {
            throw new IllegalStateException("A null URI not allowed");
        }
		List<String> list = new ArrayList<>();
		for(Map.Entry<String, String> entry : nsMap.entrySet()){
			if(entry.getValue().equals(URI)) {
                list.add(entry.getKey());
            }
		}
		return Collections.unmodifiableList(list).iterator();
    }
	/**
	 * Resolves one prefix for the given URI.
	 * @param URI
	 * @return The registered prefix 
	 */
	@Override
    public String getPrefix(String URI) {
		if(URI == null) {
            throw new IllegalStateException("A null URI not allowed");
        }
		final Iterator<String> itr = getPrefixes(URI);
		if(itr.hasNext()) {
            return itr.next();
        } else {
            return null;
        }
    }
	/**
	 * Sets a prefix to URI mapping.
	 * Overwrites existing mappings if existing, does not allow setting any of the default name space prefix mappings or mapping alternative prefixes for the default name space URI's.
	 * @param prefix
	 * @param URI
	 * @throws IllegalStateException if either argument is null.
	 */
	public void setMapping(final String prefix, final String URI){
		if(prefix==null || URI==null) {
            throw new IllegalStateException("Null prefix or URI not allowed");
        }
		if(!prefix.equals(XMLConstants.DEFAULT_NS_PREFIX) && !prefix.equals(XMLConstants.XML_NS_PREFIX) && !prefix.equals(XMLConstants.XMLNS_ATTRIBUTE)
				&& !URI.equals(XMLConstants.XML_NS_URI) && !URI.equals(XMLConstants.XML_NS_URI) && !URI.equals(XMLConstants.NULL_NS_URI)) {
            nsMap.put(prefix, URI);
        }
	}
	/**
	 * Clear user defined mappings.
	 */
	public void clearMappings(){
		nsMap.clear();
		setupDefaults();
	}
	private void setupDefaults(){
		nsMap.put(XMLConstants.XML_NS_PREFIX, XMLConstants.XML_NS_URI);
		nsMap.put(XMLConstants.XMLNS_ATTRIBUTE, XMLConstants.XMLNS_ATTRIBUTE_NS_URI);
		nsMap.put(XMLConstants.DEFAULT_NS_PREFIX, XMLConstants.NULL_NS_URI);
	}
}
